@@ -1,7 +1,7 @@
-/* $Id: milter-greylist.h,v 1.46.2.3 2006/11/07 05:12:12 manu Exp $ */
+/* $Id: milter-greylist.h,v 1.77 2008/11/26 05:20:13 manu Exp $ */
 
 /*
- * Copyright (c) 2004 Emmanuel Dreyfus
+ * Copyright (c) 2004-2007 Emmanuel Dreyfus
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,15 +37,33 @@
 #include <arpa/inet.h>
 #include <netdb.h>
 
+#ifdef USE_DKIM
+#ifdef HAVE_STDBOOL_H
+#include <stdbool.h>
+#endif
+#include <dkim.h>
+#endif
+
 #include <libmilter/mfapi.h>
 #include "config.h"
 #include "dump.h"
 
+/* environment of Solaris workaround for stdio descriptor limitation */
+#include "fd_pool.h"
+
 #define NUMLEN 20
 #define QSTRLEN 1024
 #define REGEXLEN 1024
 #define HDRLEN 1024
 #define HEADERNAME "X-Greylist"
+/* 
+ * Maximum URL length. This is just a hint, 
+ * the code will adjust the buffer if needed.
+ */
+#define URLMAXLEN	2083
+
+LIST_HEAD(urlchecklist, urlcheck_entry);
+
 
 
 #if defined(HAVE_GETNAMEINFO)
@@ -127,44 +145,110 @@ typedef union {
 
 #endif
 
+struct smtp_reply {
+	int sr_whitelist;
+	int sr_nowhitelist;
+	time_t sr_elapsed;
+	time_t sr_remaining;
+	int sr_acl_line;
+	char *sr_acl_id;
+	time_t sr_delay;
+	time_t sr_autowhite;
+	char *sr_code;
+	char *sr_ecode;
+	char *sr_msg;
+	char *sr_msg_x;
+	char *sr_report;
+	char *sr_report_x;
+	char *sr_addheader;
+	sfsistat sr_retcode;
+	int sr_nmatch;
+	char **sr_pmatch;
+};
+
+struct rcpt {
+	char r_addr[ADDRLEN + 1];
+	LIST_ENTRY(rcpt) r_list;
+};
+
+struct header {
+	char *h_line;
+	TAILQ_ENTRY(header) h_list;
+};
+
+struct body {
+	char *b_lines;
+	TAILQ_ENTRY(body) b_list;
+};
+
 struct mlfi_priv {
+	SMFICTX *priv_ctx;
 	sockaddr_t priv_addr;
 	socklen_t priv_addrlen;
 	char priv_hostname[ADDRLEN + 1];
 	char priv_helo[ADDRLEN + 1];
 	char priv_from[ADDRLEN + 1];
-	char priv_rcpt[ADDRLEN + 1];
-	time_t priv_elapsed;
-	int priv_whitelist;
+	LIST_HEAD(, rcpt) priv_rcpt;
+	char *priv_cur_rcpt;
+	int priv_rcptcount;
+	TAILQ_HEAD(, header) priv_header;
+	TAILQ_HEAD(, body) priv_body;
+#ifdef USE_GEOIP
+	const char *priv_ccode;
+#endif
+	size_t priv_msgcount;
+	char *priv_buf;
+	size_t priv_buflen;
 	char *priv_queueid;
 	int priv_delayed_reject;
-	time_t priv_remaining;
-	int priv_acl_line;
-	time_t priv_delay;
-	time_t priv_autowhite;
-	char *priv_code;
-	char *priv_ecode;
-	char *priv_msg;
+	struct smtp_reply priv_sr;
+	time_t priv_max_elapsed;
+	int priv_last_whitelist;
+#if defined(USE_CURL) || defined(USE_LDAP)
+	LIST_HEAD(, prop) priv_prop;
+#endif
+#ifdef USE_DNSRBL
+	LIST_HEAD(, dnsrbl_list) priv_dnsrbl;
+#endif
+#ifdef USE_DKIM
+	DKIM *priv_dkim;
+	DKIM_STAT priv_dkimstat;
+#endif
+#ifdef USE_P0F
+	char *priv_p0f;
+#endif
+#ifdef USE_SPAMD
+	int priv_spamd_flags;
+	int priv_spamd_score10;
+#endif
 };
 
 sfsistat mlfi_connect(SMFICTX *, char *, _SOCK_ADDR *);
 sfsistat mlfi_helo(SMFICTX *, char *);
 sfsistat mlfi_envfrom(SMFICTX *, char **);
 sfsistat mlfi_envrcpt(SMFICTX *, char **);
+sfsistat mlfi_header(SMFICTX *, char *, char *);
+sfsistat mlfi_eoh(SMFICTX *);
+sfsistat mlfi_body(SMFICTX *, unsigned char *, size_t);
 sfsistat mlfi_eom(SMFICTX *);
 sfsistat mlfi_close(SMFICTX *);
 void usage(char *);
 int humanized_atoi(char *);
+#ifndef USE_POSTFIX
+char *local_ipstr(struct mlfi_priv *);
+#endif
 struct in_addr *prefix2mask4(int, struct in_addr *);
 #ifdef AF_INET6
 struct in6_addr *prefix2mask6(int, struct in6_addr *);
 #endif
-void cleanup_sock(char *);
 void unmappedaddr(struct sockaddr *, socklen_t *);
 void final_dump(void);
 int main(int, char **);
 void mg_log(int, char *, ...);
 char *strncpy_rmsp(char *, char *, size_t);
+char *fstring_expand(struct mlfi_priv *, 
+    char *, const char *);
+char *fstring_escape(char *);
 
 #ifdef HAVE_STRLCAT
 /* #include <string.h> */
@@ -173,13 +257,6 @@ char *strncpy_rmsp(char *, char *, size_t);
 size_t mystrlcat(char *, const char *src, size_t size);
 #endif
 
-#if (defined(HAVE_SPF) || defined(HAVE_SPF_ALT))
-#define MLFI_HELO mlfi_helo
-#else
-#define MLFI_HELO NULL
-#endif
-
-
 /*
  * Locking management
  */
@@ -203,6 +280,16 @@ size_t mystrlcat(char *, const char *src, size_t size);
 	}								  \
 }
 
+#define TSS_SET(key, val) do {						  \
+	int err;							  \
+									  \
+	if ((err = pthread_setspecific(key, val)) != 0) {		  \
+		mg_log(LOG_ERR, "%s:%d pthread_setspecific failed: %s",	  \
+		    __FILE__, __LINE__, strerror(err));			  \
+		exit(EX_SOFTWARE);					  \
+	}								  \
+} while (/*CONSTCOND*/ 0)
+
 /*
  * There is a bug in GNU pth-2.0.0 that will cause a spurious EPERM
  * error when a thread releases a read lock that has been shared by
@@ -258,5 +345,17 @@ size_t mystrlcat(char *, const char *src, size_t size);
 		mystrlcat(whystr, reason, sizeof(whystr));		\
 	}
 
+/*
+ * Due to race conditions in the libmilter shipped with sendmail <= 8.13.8,
+ * the whole process may die after receiving a signal.
+ * It makes impossible the final dump. Apply the following patch ASAP:
+ * http://www.j10n.org/files/libmilter-8.13.8-signal.patch
+ *
+ * If you don't want to apply it, the following knob enables an uncertain
+ * effort to workaround the bug. Do not ask me about this.
+ * 
+ */
+/* #define WORKAROUND_LIBMILTER_RACE_CONDITION */
+
 #endif /* _MILTER_GREYLIST_H_ */
 
